using System;
using System.Globalization;
using System.Net;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using Newtonsoft.Json;


namespace RestCalculatorConsumer
{
    class Program
    {
        //public static string CalculatorUri = "https://localhost:44343/api/calculator/"; //IISExpress
        public static string CalculatorUri = "https://localhost:44389/calculator/";
        //public static string CalculatorUri = "https://localhost:5001/api/calculator/"; //Program console

       // public static string CalculatorUri = "https://miclrestcalculatorcore.azurewebsites.net/api/calculator/"; //Azure provider


        static void Main(string[] args)
        {
            Console.WriteLine("Hello World!");
            Console.ReadKey();

            Console.WriteLine("State first number; ");
            int a = Convert.ToInt32(Console.ReadLine());

            Console.WriteLine("State second number; ");
            int b = Convert.ToInt32(Console.ReadLine());

            try
            {
                Data data = new Data(a, b);

                string sumStr = (AsyncAdd(data).Result);
                Int32 sum = Convert.ToInt32(sumStr);
                Console.WriteLine("Sum converted to int:" + sum);
                Console.ReadLine();

                string diffStr = (AsyncSub(data).Result);
                Int32 diff = Convert.ToInt32(diffStr);
                Console.WriteLine("Difference converted to int: " + diff);
                Console.ReadLine();

                string mulStr = (AsyncMul(data).Result);
                Int32 prod = Convert.ToInt32(mulStr);
                Console.WriteLine("Product converted to int: " + prod);
                Console.ReadLine();

                //If you have on your PC has a Danish setup for decimal numbers 
                //i.e. commas are used as separators then one has to use utilize InvariantCulture
                CultureInfo.DefaultThreadCurrentCulture = new CultureInfo("en-EN");
                Console.WriteLine($"Current Culture: {CultureInfo.CurrentCulture.Name}");

                string divStr = (AsyncDiv(data).Result);
                Console.WriteLine("Division string to : " + divStr);
                Double div = Convert.ToDouble(divStr);
                Console.WriteLine("Division converted to double: " + div);
                Console.ReadLine();

                //If you have a Danish setup for decimal numbers i.e. commas are used as separators then
                //alternativelyy one has to can utilize InvariantCulture in TryParse
                double.TryParse(divStr, NumberStyles.Any, CultureInfo.InvariantCulture, out div);
                Console.WriteLine("Division converted to double: " + div);
                Console.ReadLine();

                string calcStr = (AsyncCalc(data, "Add").Result);
                Int32 sum1 = Convert.ToInt32(calcStr);
                Console.WriteLine("Sum converted to int: " + sum1);
                Console.ReadLine();
            }
            catch (Exception ex)
            { Console.WriteLine(ex.Message); };
        }

        public static async Task<string> AsyncAdd(Data data)
        {
            using (HttpClient client = new HttpClient())
            {
                Console.WriteLine("Data " + data);

                var jsonString = JsonConvert.SerializeObject(data);
                Console.WriteLine("json string: " + jsonString);

                StringContent content = new StringContent(jsonString, Encoding.UTF8, "application/json");
                Console.WriteLine("content: : " + content.ToString());
                Console.WriteLine("CalculatorUri: " + CalculatorUri);

                HttpResponseMessage response = await client.PostAsync(CalculatorUri + "Add", content);
                if (response.StatusCode == HttpStatusCode.Conflict)
                {
                    throw new Exception("Calculation went wrong.");
                }
                response.EnsureSuccessStatusCode();
                string str = await response.Content.ReadAsStringAsync();

                // Console.ReadLine();
                Console.WriteLine("str from method: " + str);
                //Int32 sumStr = JsonConvert.DeserializeObject<Int32>(str);

                return str;
            }
        }

        public static async Task<string> AsyncSub(Data data)
        {
            using (HttpClient client = new HttpClient())
            {
                Console.WriteLine("Data " + data);

                var jsonString = JsonConvert.SerializeObject(data);

                Console.WriteLine("json string: " + jsonString);

                StringContent content = new StringContent(jsonString, Encoding.UTF8, "application/json");

                Console.WriteLine("Content: : " + content.ToString());
                Console.WriteLine("CalculatorUri: " + CalculatorUri);

                HttpResponseMessage response = await client.PostAsync(CalculatorUri + "Sub", content);

                string str = await response.Content.ReadAsStringAsync();
                return str;

            }
        }
        public static async Task<string> AsyncMul(Data data)
        {
            using (HttpClient client = new HttpClient())
            {
                Console.WriteLine("Data " + data);

                var jsonString = JsonConvert.SerializeObject(data);
                // Console.WriteLine("json string: " + jsonString);

                StringContent content = new StringContent(jsonString, Encoding.UTF8, "application/json");

                Console.WriteLine("content: : " + content.ToString());
                Console.WriteLine("CalculatorUri: " + CalculatorUri);

                HttpResponseMessage response = await client.PostAsync(CalculatorUri + "Mul", content);
                string str = await response.Content.ReadAsStringAsync();
                return str;

            }
        }
        public static async Task<string> AsyncDiv(Data data)
        {
            using (HttpClient client = new HttpClient())
            {
                Console.WriteLine("Data " + data);

                var jsonString = JsonConvert.SerializeObject(data);
                Console.WriteLine("json string: " + jsonString);

                StringContent content = new StringContent(jsonString, Encoding.UTF8, "application/json");

                Console.WriteLine("content: : " + content.ToString());
                Console.WriteLine("CalculatorUri: " + CalculatorUri);

                HttpResponseMessage response = await client.PostAsync(CalculatorUri + "Div", content);
                if (response.StatusCode == HttpStatusCode.Conflict)
                {
                    throw new Exception("Calculation went wrong.");
                }
                response.EnsureSuccessStatusCode();


                string str = await response.Content.ReadAsStringAsync();
                return str;
            }
        }
        public static async Task<string> AsyncCalc(Data data, string calc)
        {
            using (HttpClient client = new HttpClient())
            {
                Console.WriteLine("Data " + data);

                var jsonString = JsonConvert.SerializeObject(data);
                Console.WriteLine("json string: " + jsonString);

                StringContent content = new StringContent(jsonString, Encoding.UTF8, "application/json");

                Console.WriteLine("content: : " + content.ToString());
                Console.WriteLine("CalculatorUri: " + CalculatorUri);

                HttpResponseMessage response = await client.PostAsync(CalculatorUri + calc, content);
                if (response.StatusCode == HttpStatusCode.Conflict)
                {
                    throw new Exception("Calculation went wrong.");
                }
                response.EnsureSuccessStatusCode();


                string str = await response.Content.ReadAsStringAsync();
                return str;

            }
        }
    }
}


